/*
 * Copyright (c) 2014, TU Graz, Austria.
 * Copyright (c) 2010, Swedish Institute of Computer Science (SICS), Sweden.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are not permitted.
 *
 * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ''AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Author: Carlo Alberto Boano <cboano@tugraz.at>
 *         Thiemo Voigt <thiemo@sics.se>
 *	   
 * Description: Example on how to use JamLab's interference emulation.
 *
 *
 */
 
// Interference based on empirical data collected a priori
#include <stdio.h>
#include <stdlib.h>
#include "random.h"
#include "contiki.h"
#include "node-id.h"
#include "dev/leds.h"
#include "dev/cc2420.h"
#include "dev/watchdog.h"

// Include custom libraries
//#include "settings_cc2420_interferer.h"
#include "settings_noisefloor_sampling.h"	// To carry out the noise scanning
#include "settings_cc2420_rssi.h"			// For the AGC setting
#include "settings_jamlab.h"				// To activate the interferers


/*---------------------------------------------------------------------------*/

// Channel and power
#define IEEE802154_CHANNEL_TO_INTERFERE		26
#define TXPOWER_USED_TO_INTERFERE			31

// JamLab settings
#define JAMLAB_CARRIER_TYPE 				JAMLAB_CARRIER_TYPE_MODULATED
#define JAMLAB_INTERFERENCE_TYPE			JL_CONTN

// Random seed
#define SRAND_SEED 		19


/*---------------------------------------------------------------------------*/

PROCESS(jamlab_example_process, "JamLab version 1.0, example program");
AUTOSTART_PROCESSES(&jamlab_example_process);
PROCESS_THREAD(jamlab_example_process, ev, data)
{
	PROCESS_EXITHANDLER()
	PROCESS_BEGIN();
	
	leds_off(LEDS_ALL);

	// Fixing the AGC to enable better RSSI readings  
	tune_AGC_radio(); 	
	
	// Better randomization
	srand(SRAND_SEED);		

	// Setting transmission power and radio channel
	cc2420_set_channel(IEEE802154_CHANNEL_TO_INTERFERE);
	cc2420_set_txpower(TXPOWER_USED_TO_INTERFERE);
	printf("Node: %u, PHY channel: %u, power: %u\n", node_id, IEEE802154_CHANNEL_TO_INTERFERE, TXPOWER_USED_TO_INTERFERE);
		
	// Start JamLab in emulation mode
	leds_on(LEDS_RED);
	jamlab_emulation(TXPOWER_USED_TO_INTERFERE, JAMLAB_INTERFERENCE_TYPE, JAMLAB_CARRIER_TYPE);
	
	while(1){
		PROCESS_WAIT_EVENT();
	}

	PROCESS_END();
}
