/*
 * Copyright (c) 2014, TU Graz, Austria.
 * Copyright (c) 2010, Swedish Institute of Computer Science (SICS), Sweden.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are not permitted.
 *
 * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ''AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Author: Carlo Alberto Boano <cboano@tugraz.at>
 *         Thiemo Voigt <thiemo@sics.se>
 *	   
 * Description: Example on how fast RSSI scanner building a histogram of noise. 
 *
 *
 */
 
// Interference based on empirical data collected a priori
#include <stdio.h>
#include <stdlib.h>
#include "random.h"
#include "contiki.h"
#include "node-id.h"
#include "dev/leds.h"
#include "dev/cc2420.h"
#include "dev/watchdog.h"

// Include custom libraries
//#include "settings_cc2420_interferer.h"
#include "settings_noisefloor_sampling.h"	// To carry out the noise scanning
#include "settings_cc2420_rssi.h"			// For the AGC setting


/*---------------------------------------------------------------------------*/

// Channel and power
#define IEEE802154_CHANNEL_TO_SCAN		26

// Random seed
#define SRAND_SEED 		19

// ScanLab constants
#define SCANNER_INITIAL_WAITING				(CLOCK_SECOND * 10)
#define SCANNER_NEXT_ITERATION_WAITING		(CLOCK_SECOND * 2)

// Global variables
uint32_t iteration_number = 0;


/*---------------------------------------------------------------------------*/

PROCESS(jamlab_example_process, "ScanLab version 1.0, example program");
AUTOSTART_PROCESSES(&jamlab_example_process);
PROCESS_THREAD(jamlab_example_process, ev, data)
{
	PROCESS_EXITHANDLER()
	PROCESS_BEGIN();
	
	leds_off(LEDS_ALL);

	// Fixing the AGC to enable better RSSI readings  
	tune_AGC_radio(); 	
	
	// Better randomization
	srand(SRAND_SEED);		

	// Setting transmission power and radio channel
	cc2420_set_txpower(IEEE802154_CHANNEL_TO_SCAN);
	printf("Node: %u, PHY channel: %u\n", node_id, IEEE802154_CHANNEL_TO_SCAN);
	
	// Scanning mode
	leds_on(LEDS_BLUE);
	
	// Remain temporarily inactive to allow all interferers to start (in case of testbed parallel programming)
	static struct etimer et_initial;
	etimer_set(&et_initial, SCANNER_INITIAL_WAITING);
	PROCESS_WAIT_EVENT_UNTIL(etimer_expired(&et_initial));
	
	while(1){			
		iteration_number++;
	
		// Find the noise floor
		printf("\n\nNoise floor scanning, iteration %lu:\n", iteration_number);
		find_noise_floor(IEEE802154_CHANNEL_TO_SCAN);
		
		// Just a small waiting time before next iteration
		static struct etimer et_initial;
		etimer_set(&et_initial, SCANNER_NEXT_ITERATION_WAITING);
		PROCESS_WAIT_EVENT_UNTIL(etimer_expired(&et_initial));		
	}		

	PROCESS_END();
}
